@@ -30,37 +30,51 @@ use {
     url::{Host, Url},
 };
 
-fn main() -> Result {
+fn main() {
     env_logger::Builder::from_env(
         // by default only turn on logging for agate
         env_logger::Env::default().default_filter_or("agate=info"),
     )
     .init();
-    Runtime::new()?.block_on(async {
-        let default = PresetMeta::Parameters(
-            ARGS.language
-                .as_ref()
-                .map_or(String::new(), |lang| format!(";lang={}", lang)),
-        );
-        let mimetypes = Arc::new(Mutex::new(FileOptions::new(default)));
-        let listener = TcpListener::bind(&ARGS.addrs[..]).await?;
-        log::info!("Listening on {:?}...", ARGS.addrs);
-        loop {
-            let (stream, _) = listener.accept().await?;
-            let arc = mimetypes.clone();
-            tokio::spawn(async {
-                match RequestHandle::new(stream, arc).await {
-                    Ok(handle) => match handle.handle().await {
-                        Ok(info) => log::info!("{}", info),
-                        Err(err) => log::warn!("{}", err),
-                    },
-                    Err(log_line) => {
-                        log::warn!("{}", log_line);
+    Runtime::new()
+        .expect("could not start tokio runtime")
+        .block_on(async {
+            let default = PresetMeta::Parameters(
+                ARGS.language
+                    .as_ref()
+                    .map_or(String::new(), |lang| format!(";lang={}", lang)),
+            );
+            let mimetypes = Arc::new(Mutex::new(FileOptions::new(default)));
+
+            let handles = ARGS.addrs.iter().map(|addr| {
+                let arc = mimetypes.clone();
+                tokio::spawn(async move {
+                    let listener = TcpListener::bind(addr)
+                        .await
+                        .unwrap_or_else(|e| panic!("Failed to listen on {}: {}", addr, e));
+                    log::info!("Started listener on {}", addr);
+                    loop {
+                        let (stream, _) = listener.accept().await.unwrap_or_else(|e| {
+                            panic!("could not accept new connection on {}: {}", addr, e)
+                        });
+                        let arc = arc.clone();
+                        tokio::spawn(async {
+                            match RequestHandle::new(stream, arc).await {
+                                Ok(handle) => match handle.handle().await {
+                                    Ok(info) => log::info!("{}", info),
+                                    Err(err) => log::warn!("{}", err),
+                                },
+                                Err(log_line) => {
+                                    log::warn!("{}", log_line);
+                                }
+                            }
+                        });
                     }
-                }
+                })
             });
-        }
-    })
+
+            futures_util::future::join_all(handles).await;
+        });
 }
 
 type Result<T = (), E = Box<dyn Error + Send + Sync>> = std::result::Result<T, E>;
@@ -82,6 +96,7 @@ struct Args {
     log_ips: bool,
     only_tls13: bool,
     central_config: bool,
+    skip_port_check: bool,
 }
 
 fn args() -> Result<Args> {
@@ -140,6 +155,11 @@ fn args() -> Result<Args> {
         "ed25519",
         "Generate keys using the Ed25519 signature algorithm instead of the default ECDSA.",
     );
+    opts.optflag(
+        "",
+        "skip-port-check",
+        "Skip URL port check even when a hostname is specified.",
+    );
 
     let matches = opts.parse(&args[1..]).map_err(|f| f.to_string())?;
 
@@ -263,6 +283,7 @@ fn args() -> Result<Args> {
         log_ips: matches.opt_present("log-ip"),
         only_tls13: matches.opt_present("only-tls13"),
         central_config: matches.opt_present("central-conf"),
+        skip_port_check: matches.opt_present("skip-port-check"),
     })
 }
 
@@ -413,10 +434,13 @@ impl RequestHandle {
         }
 
         // correct port
-        if let Some(port) = url.port() {
-            // Validate that the port in the URL is the same as for the stream this request came in on.
-            if port != self.stream.get_ref().0.local_addr().unwrap().port() {
-                return Err((53, "proxy request refused"));
+        if !ARGS.skip_port_check {
+            if let Some(port) = url.port() {
+                // Validate that the port in the URL is the same as for the stream this request
+                // came in on.
+                if port != self.stream.get_ref().0.local_addr().unwrap().port() {
+                    return Err((53, "Proxy request refused"));
+                }
             }
         }
         Ok(url)
